

/*
 *
 * this file is part of the "cdmon" project
 * Copyright (C) 2004 by Jean Philippe GUILLEMIN <jp.guillemin@free.fr>
 * license: This software is under GPL license
 * rev: 0.41
 */

#include "cdmon.h"


/***************** get_event_name_from_id ********************/
/* get_event_name_from_id() - return printable name of fam event code */
const char *get_event_name_from_id(int code)
{
    static const char *famevent[] = {
        "",
        "FAMChanged",
        "FAMDeleted",
        "FAMStartExecuting",
        "FAMStopExecuting",
        "FAMCreated",
        "FAMMoved",
        "FAMAcknowledge",
        "FAMExists",
        "FAMEndExist"
    };
    static char unknown_event[10];
 
    if (code < FAMChanged || code > FAMEndExist)
    {
        sprintf(unknown_event, "unknown (%d)", code);
        return unknown_event;
    }
    return famevent[code];
}

/***************** famdetect ********************/
/* famdetect() - detect creation of device and forks dockapp  */
int famdetect(int nbdev, char * device[], char *event) {
    pid_t	child, pid;
    int i, j, rc, fam_fd, output, status;
    FAMConnection fc;
    FAMRequest *frp;
    FAMEvent fe;
    fd_set readfds;
 	char *proc_cd;
	FILE *file;
	proc_cd = malloc(MEDIUM_STRING);
	//fprintf(stderr, "%s\n", event); //debug    
		char *buffer1[nbdev], *buffer2[nbdev];
		for (i = 0; i < nbdev; i++){
			buffer1[i] = malloc(SHORT_STRING);
			buffer2[i] = malloc(SHORT_STRING);
			strncpy (buffer2[i], "0", SHORT_STRING);
		}
	
	
/* Allocate storage for requests */
    frp = malloc(nbdev * sizeof *frp);
    if (!frp) {
       fprintf(stderr, "malloc failed in famdetect()\n");
        exit(1);
    }
 
    /* Open fam connection */
 
    if ((FAMOpen(&fc)) < 0) {
        fprintf(stderr, "fam failed in famdetect()\n");
        exit(1);
    }
 
    /* Request monitoring for each program argument */

    for ( i = 1; i < nbdev; i++) {
		//fprintf(stderr, "%s\n", device[i]); //debug
		rc = FAMMonitorFile(&fc, device[i], frp + i, NULL);
		if (rc < 0) {
			fprintf(stderr, "FAMMonitor failed in famdetect()\n");
			continue;
		}
	}
 
    /* Initialize select data structure */
 
    fam_fd = FAMCONNECTION_GETFD(&fc);
    FD_ZERO(&readfds);
    FD_SET(fam_fd, &readfds);

/* Loop forever. */

	while(1){
		if (select(fam_fd + 1, &readfds,
				   NULL, NULL, NULL) < 0){
			 fprintf(stderr, "select failed in famdetect()\n");
			 exit(1);
		}
		if (FD_ISSET(fam_fd, &readfds)){
			if (FAMNextEvent(&fc, &fe) < 0){
				fprintf(stderr, "FAMNextEvent failed in famdetect()\n");
				exit(1);
			}
			if (! strcmp(get_event_name_from_id(fe.code), event)){
				strncpy(proc_cd, fe.filename, MEDIUM_STRING);
				for (j = 0; j < nbdev; j++){
					if (! strcmp(proc_cd, device[j])) break;
				}
				fprintf(stderr, "capacityfile = %s\n", proc_cd); // debug
				if((file=fopen(proc_cd, "rb")) == 0) fprintf (stderr,"Error opening file %s !\n", proc_cd);
				memset(buffer1[j], 0, SHORT_STRING);
				fgets (buffer1[j], SHORT_STRING, file);
				if (!(strncmp ("0", buffer1[j], 1))){
					memset(buffer2[j], 0, SHORT_STRING);
					strncpy (buffer2[j], buffer1[j], SHORT_STRING);
				}
										fprintf(stderr, "bufer1 = %s\n", buffer1[j]); // debug
										fprintf(stderr, "bufer2 = %s\n", buffer2[j]); // debug
				fclose (file);
				if (((atoi(buffer1[j])) > 10)&& (!(strncmp ("0", buffer2[j], 1)))) {
					memset(buffer2[j], 0, SHORT_STRING);
					strncpy (buffer2[j], buffer1[j], SHORT_STRING);

					pid = 0;
					get_event_name_from_id(fe.code);
					signal (SIGCHLD, SIG_IGN);
					switch (child=fork()) {
						case -1: fprintf(stderr, "fork error in famdetect()\n");   /* Erreur */
							 break;
						case 0:
							
							if ((output  = dockapp(proc_cd, nbdev, device))){
								fprintf(stderr, "error launching dockapp in famdetect()\n");
							}
							break;
						default:
							while (1) {
								pid = waitpid (WAIT_ANY, &status, WNOHANG);
								if (pid < 0) {
									  fprintf(stderr, "error waitpid in famdetect()\n");
									  break;
								}
								if (pid == 0) break;
							}						
							break;
					}
				}
			}	
		}
	}
free(proc_cd);
free(buffer1);
free(buffer2);
return EXIT_SUCCESS;
}
